"""Methods to convert to and from waveform protobuf messages."""

from __future__ import annotations

import datetime as dt
from collections.abc import Mapping
from typing import Any

import hightime as ht
import nitypes.bintime as bt
import numpy as np
from nitypes.complex import ComplexInt32Base, ComplexInt32DType
from nitypes.time import convert_datetime
from nitypes.waveform import (
    AnalogWaveform,
    ComplexWaveform,
    DigitalWaveform,
    ExtendedPropertyDictionary,
    LinearScaleMode,
    NoneScaleMode,
    SampleIntervalMode,
    Spectrum,
    Timing,
)
from nitypes.waveform.typing import ExtendedPropertyValue

import ni.protobuf.types.precision_timestamp_conversion as ptc
from ni.protobuf.types.precision_timestamp_pb2 import PrecisionTimestamp
from ni.protobuf.types.waveform_pb2 import (
    DigitalWaveform as DigitalWaveformProto,
    DoubleAnalogWaveform,
    DoubleComplexWaveform,
    DoubleSpectrum,
    I16AnalogWaveform,
    I16ComplexWaveform,
    LinearScale,
    Scale,
    WaveformAttributeValue,
)


def float64_analog_waveform_to_protobuf(
    value: AnalogWaveform[np.float64], /
) -> DoubleAnalogWaveform:
    """Convert the Python AnalogWaveform to a protobuf DoubleAnalogWaveform."""
    _validate_timing(value)
    t0 = _t0_from_waveform(value)
    time_interval = _time_interval_from_waveform(value)
    attributes = _extended_properties_to_attributes(value.extended_properties)

    return DoubleAnalogWaveform(
        t0=t0,
        dt=time_interval,
        y_data=value.scaled_data,
        attributes=attributes,
    )


def float64_analog_waveform_from_protobuf(
    message: DoubleAnalogWaveform, /
) -> AnalogWaveform[np.float64]:
    """Convert the protobuf DoubleAnalogWaveform to a Python AnalogWaveform."""
    timing = _timing_from_waveform_message(message)
    extended_properties = _attributes_to_extended_properties(message.attributes)

    return AnalogWaveform.from_array_1d(
        message.y_data,
        dtype=np.float64,
        extended_properties=extended_properties,
        timing=timing,
        scale_mode=NoneScaleMode(),
    )


def float64_complex_waveform_to_protobuf(
    value: ComplexWaveform[np.complex128], /
) -> DoubleComplexWaveform:
    """Convert the Python ComplexWaveform to a protobuf DoubleComplexWaveform."""
    _validate_timing(value)
    t0 = _t0_from_waveform(value)
    time_interval = _time_interval_from_waveform(value)
    attributes = _extended_properties_to_attributes(value.extended_properties)

    interleaved_array = value.scaled_data.view(np.float64)

    return DoubleComplexWaveform(
        t0=t0,
        dt=time_interval,
        y_data=interleaved_array,
        attributes=attributes,
    )


def float64_complex_waveform_from_protobuf(
    message: DoubleComplexWaveform, /
) -> ComplexWaveform[np.complex128]:
    """Convert the protobuf DoubleComplexWaveform to a Python ComplexWaveform."""
    timing = _timing_from_waveform_message(message)
    extended_properties = _attributes_to_extended_properties(message.attributes)

    y_array = np.array(message.y_data, np.float64)
    data_array = y_array.view(np.complex128)

    return ComplexWaveform.from_array_1d(
        data_array,
        copy=False,
        extended_properties=extended_properties,
        timing=timing,
        scale_mode=NoneScaleMode(),
    )


def int16_complex_waveform_to_protobuf(
    value: ComplexWaveform[ComplexInt32Base], /
) -> I16ComplexWaveform:
    """Convert the Python ComplexWaveform to a protobuf DoubleComplexWaveform."""
    _validate_timing(value)
    t0 = _t0_from_waveform(value)
    time_interval = _time_interval_from_waveform(value)
    attributes = _extended_properties_to_attributes(value.extended_properties)
    scale = _scale_from_waveform(value)

    interleaved_array = value.raw_data.view(np.int16)

    return I16ComplexWaveform(
        t0=t0,
        dt=time_interval,
        y_data=interleaved_array,
        attributes=attributes,
        scale=scale,
    )


def int16_complex_waveform_from_protobuf(
    message: I16ComplexWaveform, /
) -> ComplexWaveform[ComplexInt32Base]:
    """Convert the protobuf DoubleComplexWaveform to a Python ComplexWaveform."""
    timing = _timing_from_waveform_message(message)
    extended_properties = _attributes_to_extended_properties(message.attributes)
    scale_mode = _scale_mode_from_waveform_message(message)

    y_array = np.array(message.y_data, np.int16)
    data_array = y_array.view(ComplexInt32DType)

    return ComplexWaveform.from_array_1d(
        data_array,
        copy=False,
        extended_properties=extended_properties,
        timing=timing,
        scale_mode=scale_mode,
    )


def float64_spectrum_to_protobuf(value: Spectrum[np.float64], /) -> DoubleSpectrum:
    """Convert the Python Spectrum to a protobuf DoubleSpectrum."""
    attributes = _extended_properties_to_attributes(value.extended_properties)
    return DoubleSpectrum(
        start_frequency=value.start_frequency,
        frequency_increment=value.frequency_increment,
        data=value.data,
        attributes=attributes,
    )


def float64_spectrum_from_protobuf(message: DoubleSpectrum, /) -> Spectrum[np.float64]:
    """Convert the protobuf DoubleSpectrum to a Python Spectrum."""
    extended_properties = _attributes_to_extended_properties(message.attributes)
    return Spectrum.from_array_1d(
        message.data,
        dtype=np.float64,
        start_frequency=message.start_frequency,
        frequency_increment=message.frequency_increment,
        extended_properties=extended_properties,
    )


def int16_analog_waveform_to_protobuf(value: AnalogWaveform[np.int16], /) -> I16AnalogWaveform:
    """Convert the Python AnalogWaveform to a protobuf I16AnalogWaveform."""
    _validate_timing(value)
    t0 = _t0_from_waveform(value)
    time_interval = _time_interval_from_waveform(value)
    attributes = _extended_properties_to_attributes(value.extended_properties)
    scale = _scale_from_waveform(value)

    return I16AnalogWaveform(
        t0=t0,
        dt=time_interval,
        y_data=value.raw_data,
        attributes=attributes,
        scale=scale,
    )


def int16_analog_waveform_from_protobuf(message: I16AnalogWaveform, /) -> AnalogWaveform[np.int16]:
    """Convert the protobuf I16AnalogWaveform to a Python AnalogWaveform."""
    timing = _timing_from_waveform_message(message)
    extended_properties = _attributes_to_extended_properties(message.attributes)
    scale_mode = _scale_mode_from_waveform_message(message)

    return AnalogWaveform.from_array_1d(
        message.y_data,
        dtype=np.int16,
        extended_properties=extended_properties,
        timing=timing,
        scale_mode=scale_mode,
    )


def digital_waveform_to_protobuf(value: DigitalWaveform[Any], /) -> DigitalWaveformProto:
    """Convert the Python DigitalWaveform to a protobuf DigitalWaveform."""
    _validate_timing(value)
    t0 = _t0_from_waveform(value)
    time_interval = _time_interval_from_waveform(value)
    attributes = _extended_properties_to_attributes(value.extended_properties)

    return DigitalWaveformProto(
        t0=t0,
        dt=time_interval,
        signal_count=value.signal_count,
        y_data=value.data.tobytes(),
        attributes=attributes,
    )


def digital_waveform_from_protobuf(message: DigitalWaveformProto, /) -> DigitalWaveform[np.uint8]:
    """Convert the protobuf DigitalWaveform to a Python DigitalWaveform."""
    timing = _timing_from_waveform_message(message)
    extended_properties = _attributes_to_extended_properties(message.attributes)

    if message.signal_count <= 0:
        raise ValueError("signal_count must be greater than zero.")

    data_array = np.frombuffer(message.y_data, dtype=np.uint8)
    samples_per_signal = len(data_array) // message.signal_count
    if len(data_array) != samples_per_signal * message.signal_count:
        raise ValueError(f"Data array length ({len(data_array)}) does not match expected shape.")
    reshaped_data = data_array.reshape(samples_per_signal, message.signal_count)

    return DigitalWaveform.from_lines(
        reshaped_data,
        dtype=np.uint8,
        signal_count=message.signal_count,
        extended_properties=extended_properties,
        timing=timing,
    )


def _attributes_to_extended_properties(
    attributes: Mapping[str, WaveformAttributeValue],
) -> Mapping[str, ExtendedPropertyValue]:
    extended_properties = {}
    for key, value in attributes.items():
        attr_type = value.WhichOneof("attribute")
        if attr_type is None:
            raise ValueError("Could not determine the datatype of 'attribute'.")
        extended_properties[key] = getattr(value, attr_type)

    return extended_properties


def _extended_properties_to_attributes(
    extended_properties: ExtendedPropertyDictionary,
) -> Mapping[str, WaveformAttributeValue]:
    return {key: _value_to_attribute(value) for key, value in extended_properties.items()}


def _value_to_attribute(value: ExtendedPropertyValue) -> WaveformAttributeValue:
    attr_value = WaveformAttributeValue()
    if isinstance(value, bool):
        attr_value.bool_value = value
    elif isinstance(value, int):
        attr_value.integer_value = value
    elif isinstance(value, float):
        attr_value.double_value = value
    elif isinstance(value, str):
        attr_value.string_value = value
    else:
        raise TypeError(f"Unexpected type for extended property value {type(value)}")

    return attr_value


def _validate_timing(
    waveform: AnalogWaveform[Any] | ComplexWaveform[Any] | DigitalWaveform[Any],
) -> None:
    if waveform.timing.sample_interval_mode == SampleIntervalMode.IRREGULAR:
        raise ValueError("Cannot convert irregular sample interval to protobuf.")


def _t0_from_waveform(
    waveform: AnalogWaveform[Any] | ComplexWaveform[Any] | DigitalWaveform[Any],
) -> PrecisionTimestamp | None:
    if waveform.timing.has_start_time:
        bin_datetime = convert_datetime(bt.DateTime, waveform.timing.start_time)
        return ptc.bintime_datetime_to_protobuf(bin_datetime)
    else:
        return None


def _time_interval_from_waveform(
    waveform: AnalogWaveform[Any] | ComplexWaveform[Any] | DigitalWaveform[Any],
) -> float:
    if waveform.timing.has_sample_interval:
        return waveform.timing.sample_interval.total_seconds()
    else:
        return 0


def _timing_from_waveform_message(
    message: (
        DoubleAnalogWaveform
        | DoubleComplexWaveform
        | I16AnalogWaveform
        | I16ComplexWaveform
        | DigitalWaveformProto
    ),
) -> Timing[bt.DateTime | dt.datetime]:
    # Declare timing to accept both bintime and dt.datetime to satisfy mypy.
    timing: Timing[bt.DateTime | dt.datetime]
    if not message.dt and not message.HasField("t0"):
        # If both dt and t0 are unset, use Timing.empty.
        timing = Timing.empty
    else:
        # Timestamp
        bin_datetime = ptc.bintime_datetime_from_protobuf(message.t0)

        # Sample Interval
        if not message.dt:
            timing = Timing.create_with_no_interval(timestamp=bin_datetime)
        else:
            sample_interval = ht.timedelta(seconds=message.dt)
            timing = Timing.create_with_regular_interval(
                sample_interval=sample_interval,
                timestamp=bin_datetime,
            )

    return timing


def _scale_from_waveform(waveform: AnalogWaveform[Any] | ComplexWaveform[Any]) -> Scale | None:
    if isinstance(waveform.scale_mode, LinearScaleMode):
        linear_scale = LinearScale(gain=waveform.scale_mode.gain, offset=waveform.scale_mode.offset)
        return Scale(linear_scale=linear_scale)
    elif isinstance(waveform.scale_mode, NoneScaleMode):
        return None
    else:
        raise ValueError(f"The waveform scale mode {waveform.scale_mode} is not supported.")


def _scale_mode_from_waveform_message(
    message: I16AnalogWaveform | I16ComplexWaveform,
) -> LinearScaleMode | NoneScaleMode:
    if message.HasField("scale"):
        mode = message.scale.WhichOneof("mode")
        if mode is None:
            raise ValueError("Could not determine waveform scale mode.")
        elif mode == "linear_scale":
            return LinearScaleMode(
                message.scale.linear_scale.gain, message.scale.linear_scale.offset
            )
        else:
            raise ValueError(f"The waveform scale mode {mode!r} is not supported.")

    return NoneScaleMode()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ni/protobuf/types/waveform_wrappers.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ni.protobuf.types import waveform_pb2 as ni_dot_protobuf_dot_types_dot_waveform__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)ni/protobuf/types/waveform_wrappers.proto\x12\x11ni.protobuf.types\x1a ni/protobuf/types/waveform.proto\"\\\n\x1e\x44oubleAnalogWaveformArrayValue\x12:\n\twaveforms\x18\x01 \x03(\x0b\x32\'.ni.protobuf.types.DoubleAnalogWaveform\"V\n\x1bI16AnalogWaveformArrayValue\x12\x37\n\twaveforms\x18\x01 \x03(\x0b\x32$.ni.protobuf.types.I16AnalogWaveform\"^\n\x1f\x44oubleComplexWaveformArrayValue\x12;\n\twaveforms\x18\x01 \x03(\x0b\x32(.ni.protobuf.types.DoubleComplexWaveform\"X\n\x1cI16ComplexWaveformArrayValue\x12\x38\n\twaveforms\x18\x01 \x03(\x0b\x32%.ni.protobuf.types.I16ComplexWaveform\"P\n\x18\x44oubleSpectrumArrayValue\x12\x34\n\twaveforms\x18\x01 \x03(\x0b\x32!.ni.protobuf.types.DoubleSpectrumB\x8d\x01\n\x15\x63om.ni.protobuf.typesB\x15WaveformWrappersProtoP\x01Z\x05types\xa2\x02\x04NIPT\xaa\x02\"NationalInstruments.Protobuf.Types\xca\x02\x11NI\\PROTOBUF\\TYPES\xea\x02\x13NI::Protobuf::Typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ni.protobuf.types.waveform_wrappers_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.ni.protobuf.typesB\025WaveformWrappersProtoP\001Z\005types\242\002\004NIPT\252\002\"NationalInstruments.Protobuf.Types\312\002\021NI\\PROTOBUF\\TYPES\352\002\023NI::Protobuf::Types'
  _DOUBLEANALOGWAVEFORMARRAYVALUE._serialized_start=98
  _DOUBLEANALOGWAVEFORMARRAYVALUE._serialized_end=190
  _I16ANALOGWAVEFORMARRAYVALUE._serialized_start=192
  _I16ANALOGWAVEFORMARRAYVALUE._serialized_end=278
  _DOUBLECOMPLEXWAVEFORMARRAYVALUE._serialized_start=280
  _DOUBLECOMPLEXWAVEFORMARRAYVALUE._serialized_end=374
  _I16COMPLEXWAVEFORMARRAYVALUE._serialized_start=376
  _I16COMPLEXWAVEFORMARRAYVALUE._serialized_end=464
  _DOUBLESPECTRUMARRAYVALUE._serialized_start=466
  _DOUBLESPECTRUMARRAYVALUE._serialized_end=546
# @@protoc_insertion_point(module_scope)



"""Methods to convert to and from PrecisionTimestamp protobuf messages."""

from __future__ import annotations

import hightime as ht
import nitypes.bintime as bt
from nitypes.time import convert_datetime

from ni.protobuf.types.precision_timestamp_pb2 import (
    PrecisionTimestamp,
)


def bintime_datetime_to_protobuf(value: bt.DateTime, /) -> PrecisionTimestamp:
    """Convert a NI-BTF DateTime to a protobuf PrecisionTimestamp."""
    seconds, fractional_seconds = value.to_tuple()
    return PrecisionTimestamp(seconds=seconds, fractional_seconds=fractional_seconds)


def bintime_datetime_from_protobuf(message: PrecisionTimestamp, /) -> bt.DateTime:
    """Convert a protobuf PrecisionTimestamp to a NI-BTF DateTime."""
    time_value_tuple = bt.TimeValueTuple(message.seconds, message.fractional_seconds)
    return bt.DateTime.from_tuple(time_value_tuple)


def hightime_datetime_to_protobuf(value: ht.datetime, /) -> PrecisionTimestamp:
    """Convert a hightime.datetime to a protobuf PrecisionTimestamp."""
    bt_datetime = convert_datetime(bt.DateTime, value)
    return bintime_datetime_to_protobuf(bt_datetime)


def hightime_datetime_from_protobuf(message: PrecisionTimestamp, /) -> ht.datetime:
    """Convert a protobuf PrecisionTimestamp to a hightime.datetime."""
    bt_datetime = bintime_datetime_from_protobuf(message)
    ht_datetime = convert_datetime(ht.datetime, bt_datetime)
    return ht_datetime



"""Methods to convert to and from scalar protobuf messages."""

from __future__ import annotations

from typing import Any, cast

from nitypes.vector import Vector

import ni.protobuf.types.array_pb2 as array_pb2
import ni.protobuf.types.vector_pb2 as vector_pb2
from ni.protobuf.types.attribute_value_pb2 import AttributeValue
from ni.protobuf.types.extended_property_conversion import (
    extended_properties_from_protobuf,
    extended_properties_to_protobuf,
)
from ni.protobuf.types.scalar_conversion import AnyScalarType

_VECTOR_TYPE_TO_PB_ATTR_MAP = {
    bool: "bool_array",
    int: "sint32_array",
    float: "double_array",
    str: "string_array",
}


def vector_to_protobuf(value: Vector[Any], /) -> vector_pb2.Vector:
    """Convert a Vector python object to a protobuf vector_pb2.Vector."""
    if not len(value):
        raise ValueError("Cannot convert an empty vector.")

    _check_vector_values(value)
    attributes = extended_properties_to_protobuf(value.extended_properties)
    return _create_vector_message(value, attributes)


def vector_from_protobuf(message: vector_pb2.Vector, /) -> Vector[AnyScalarType]:
    """Convert the protobuf vector_pb2.Vector to a Python Vector."""
    pb_type = message.WhichOneof("value")
    if pb_type is None:
        raise ValueError("Could not determine the data type of 'value'.")

    if pb_type not in _VECTOR_TYPE_TO_PB_ATTR_MAP.values():
        raise ValueError(f"Unexpected value for protobuf_value.WhichOneOf: {pb_type}")

    value: (
        array_pb2.BoolArray | array_pb2.SInt32Array | array_pb2.DoubleArray | array_pb2.StringArray
    )
    value = getattr(message, pb_type)

    # Create with blank units. Units from the protobuf message will be populated
    # when attributes are converted to an ExtendedPropertyDictionary.
    vector = Vector(value.values, "")

    # Transfer attributes to extended_properties
    extended_properties_from_protobuf(message.attributes, vector.extended_properties)

    return vector


def _create_vector_message(
    vector_obj: Vector[Any],
    attributes: dict[str, AttributeValue],
) -> vector_pb2.Vector:
    if isinstance(vector_obj[0], bool):
        bool_vector = cast(Vector[bool], vector_obj)
        bool_array = array_pb2.BoolArray(values=bool_vector)
        return vector_pb2.Vector(attributes=attributes, bool_array=bool_array)
    elif isinstance(vector_obj[0], int):
        int_vector = cast(Vector[int], vector_obj)
        int_array = array_pb2.SInt32Array(values=int_vector)
        return vector_pb2.Vector(attributes=attributes, sint32_array=int_array)
    elif isinstance(vector_obj[0], float):
        double_vector = cast(Vector[float], vector_obj)
        double_array = array_pb2.DoubleArray(values=double_vector)
        return vector_pb2.Vector(attributes=attributes, double_array=double_array)
    elif isinstance(vector_obj[0], str):
        string_vector = cast(Vector[str], vector_obj)
        string_array = array_pb2.StringArray(values=string_vector)
        return vector_pb2.Vector(attributes=attributes, string_array=string_array)
    else:
        raise TypeError(f"Invalid array value type: {type(vector_obj[0])}")


def _check_vector_values(vector: Vector[AnyScalarType]) -> None:
    for value in vector:
        if isinstance(value, int):
            if value <= -0x80000000 or value >= 0x7FFFFFFF:
                raise ValueError("Integer values in a vector must be within the range of an Int32.")



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
---------------------------------------------------------------------
---------------------------------------------------------------------
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import ni.protobuf.types.precision_timestamp_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class DoubleAnalogWaveform(google.protobuf.message.Message):
    """A double-precision analog waveform with timing and extended properties."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttributesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___WaveformAttributeValue: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___WaveformAttributeValue | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    T0_FIELD_NUMBER: builtins.int
    DT_FIELD_NUMBER: builtins.int
    Y_DATA_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    dt: builtins.float
    """The time interval in seconds between data points in the waveform."""
    @property
    def t0(self) -> ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp:
        """The time of the first sample in y_data."""

    @property
    def y_data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]:
        """The data values of the waveform."""

    @property
    def attributes(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___WaveformAttributeValue]:
        """Attribute names and values. See WaveformAttributeValue for more details."""

    def __init__(
        self,
        *,
        t0: ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp | None = ...,
        dt: builtins.float = ...,
        y_data: collections.abc.Iterable[builtins.float] | None = ...,
        attributes: collections.abc.Mapping[builtins.str, global___WaveformAttributeValue] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["t0", b"t0"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attributes", b"attributes", "dt", b"dt", "t0", b"t0", "y_data", b"y_data"]) -> None: ...

global___DoubleAnalogWaveform = DoubleAnalogWaveform

@typing.final
class I16AnalogWaveform(google.protobuf.message.Message):
    """A 16-bit integer analog waveform with timing and extended properties."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttributesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___WaveformAttributeValue: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___WaveformAttributeValue | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    T0_FIELD_NUMBER: builtins.int
    DT_FIELD_NUMBER: builtins.int
    Y_DATA_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    SCALE_FIELD_NUMBER: builtins.int
    dt: builtins.float
    """The time interval in seconds between data points in the waveform."""
    @property
    def t0(self) -> ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp:
        """The time of the first sample in y_data."""

    @property
    def y_data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """The data values of the waveform."""

    @property
    def attributes(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___WaveformAttributeValue]:
        """Attribute names and values. See WaveformAttributeValue for more details."""

    @property
    def scale(self) -> global___Scale:
        """Optional scaling information used to convert raw data to scaled data."""

    def __init__(
        self,
        *,
        t0: ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp | None = ...,
        dt: builtins.float = ...,
        y_data: collections.abc.Iterable[builtins.int] | None = ...,
        attributes: collections.abc.Mapping[builtins.str, global___WaveformAttributeValue] | None = ...,
        scale: global___Scale | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["scale", b"scale", "t0", b"t0"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attributes", b"attributes", "dt", b"dt", "scale", b"scale", "t0", b"t0", "y_data", b"y_data"]) -> None: ...

global___I16AnalogWaveform = I16AnalogWaveform

@typing.final
class DoubleComplexWaveform(google.protobuf.message.Message):
    """A double-precision complex waveform with timing and extended properties."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttributesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___WaveformAttributeValue: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___WaveformAttributeValue | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    T0_FIELD_NUMBER: builtins.int
    DT_FIELD_NUMBER: builtins.int
    Y_DATA_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    dt: builtins.float
    """The time interval in seconds between data points in the waveform."""
    @property
    def t0(self) -> ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp:
        """The time of the first sample in y_data."""

    @property
    def y_data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]:
        """The data values of the waveform.

        This data consists of interleaved real and imaginary parts.
        Example: [1.0+2.0j, 3.0+4.0j] is represented as [1.0, 2.0, 3.0, 4.0].
        """

    @property
    def attributes(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___WaveformAttributeValue]:
        """Attribute names and values. See WaveformAttributeValue for more details."""

    def __init__(
        self,
        *,
        t0: ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp | None = ...,
        dt: builtins.float = ...,
        y_data: collections.abc.Iterable[builtins.float] | None = ...,
        attributes: collections.abc.Mapping[builtins.str, global___WaveformAttributeValue] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["t0", b"t0"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attributes", b"attributes", "dt", b"dt", "t0", b"t0", "y_data", b"y_data"]) -> None: ...

global___DoubleComplexWaveform = DoubleComplexWaveform

@typing.final
class I16ComplexWaveform(google.protobuf.message.Message):
    """A 16-bit integer complex waveform with timing and extended properties."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttributesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___WaveformAttributeValue: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___WaveformAttributeValue | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    T0_FIELD_NUMBER: builtins.int
    DT_FIELD_NUMBER: builtins.int
    Y_DATA_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    SCALE_FIELD_NUMBER: builtins.int
    dt: builtins.float
    """The time interval in seconds between data points in the waveform."""
    @property
    def t0(self) -> ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp:
        """The time of the first sample in y_data."""

    @property
    def y_data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """The data values of the waveform.

        This data consists of interleaved real and imaginary parts.
        Example: [1+2j, 3+4j] is represented as [1, 2, 3, 4].
        """

    @property
    def attributes(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___WaveformAttributeValue]:
        """Attribute names and values. See WaveformAttributeValue for more details."""

    @property
    def scale(self) -> global___Scale:
        """Optional scaling information used to convert raw data to scaled data."""

    def __init__(
        self,
        *,
        t0: ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp | None = ...,
        dt: builtins.float = ...,
        y_data: collections.abc.Iterable[builtins.int] | None = ...,
        attributes: collections.abc.Mapping[builtins.str, global___WaveformAttributeValue] | None = ...,
        scale: global___Scale | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["scale", b"scale", "t0", b"t0"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attributes", b"attributes", "dt", b"dt", "scale", b"scale", "t0", b"t0", "y_data", b"y_data"]) -> None: ...

global___I16ComplexWaveform = I16ComplexWaveform

@typing.final
class DoubleSpectrum(google.protobuf.message.Message):
    """A double-precision frequency spectrum with extended properties."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttributesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___WaveformAttributeValue: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___WaveformAttributeValue | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    START_FREQUENCY_FIELD_NUMBER: builtins.int
    FREQUENCY_INCREMENT_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    start_frequency: builtins.float
    """The start frequency of the spectrum."""
    frequency_increment: builtins.float
    """The frequency increment of the spectrum."""
    @property
    def data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]:
        """The data values of the spectrum."""

    @property
    def attributes(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___WaveformAttributeValue]:
        """Attribute names and values. See WaveformAttributeValue for more details."""

    def __init__(
        self,
        *,
        start_frequency: builtins.float = ...,
        frequency_increment: builtins.float = ...,
        data: collections.abc.Iterable[builtins.float] | None = ...,
        attributes: collections.abc.Mapping[builtins.str, global___WaveformAttributeValue] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["attributes", b"attributes", "data", b"data", "frequency_increment", b"frequency_increment", "start_frequency", b"start_frequency"]) -> None: ...

global___DoubleSpectrum = DoubleSpectrum

@typing.final
class WaveformAttributeValue(google.protobuf.message.Message):
    """Metadata attached to a waveform.

    The NI-DAQmx driver sets the following string attributes:

    - NI_ChannelName: the name of the virtual channel producing the waveform.
    - NI_LineNames: the name of the digital line in the waveform.
    - NI_UnitDescription: the units of measure for the waveform.
    - NI_dBReference: the reference value to use when converting measurement levels to decibel.

    For additional information on waveform attributes, please visit
    https://www.ni.com/docs/en-US/bundle/labview-api-ref/page/functions/get-waveform-attribute.html
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BOOL_VALUE_FIELD_NUMBER: builtins.int
    INTEGER_VALUE_FIELD_NUMBER: builtins.int
    DOUBLE_VALUE_FIELD_NUMBER: builtins.int
    STRING_VALUE_FIELD_NUMBER: builtins.int
    bool_value: builtins.bool
    """Represents a bool attribute."""
    integer_value: builtins.int
    """Represents an integer attribute."""
    double_value: builtins.float
    """Represents a double attribute."""
    string_value: builtins.str
    """Represents a string attribute."""
    def __init__(
        self,
        *,
        bool_value: builtins.bool = ...,
        integer_value: builtins.int = ...,
        double_value: builtins.float = ...,
        string_value: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["attribute", b"attribute", "bool_value", b"bool_value", "double_value", b"double_value", "integer_value", b"integer_value", "string_value", b"string_value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attribute", b"attribute", "bool_value", b"bool_value", "double_value", b"double_value", "integer_value", b"integer_value", "string_value", b"string_value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["attribute", b"attribute"]) -> typing.Literal["bool_value", "integer_value", "double_value", "string_value"] | None: ...

global___WaveformAttributeValue = WaveformAttributeValue

@typing.final
class DigitalWaveform(google.protobuf.message.Message):
    """A digital waveform as bytes with timing and extended properties."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttributesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___WaveformAttributeValue: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___WaveformAttributeValue | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    T0_FIELD_NUMBER: builtins.int
    DT_FIELD_NUMBER: builtins.int
    SIGNAL_COUNT_FIELD_NUMBER: builtins.int
    Y_DATA_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    dt: builtins.float
    """The time interval in seconds between data points in the waveform."""
    signal_count: builtins.int
    """The number of signals in each sample of data."""
    y_data: builtins.bytes
    """The data values of the waveform.

    This data is a flattened array of bytes that are ordered such that each
    signal_count bytes represents a sample.
    """
    @property
    def t0(self) -> ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp:
        """The time of the first sample in y_data."""

    @property
    def attributes(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___WaveformAttributeValue]:
        """Attribute names and values. See WaveformAttributeValue for more details."""

    def __init__(
        self,
        *,
        t0: ni.protobuf.types.precision_timestamp_pb2.PrecisionTimestamp | None = ...,
        dt: builtins.float = ...,
        signal_count: builtins.int = ...,
        y_data: builtins.bytes = ...,
        attributes: collections.abc.Mapping[builtins.str, global___WaveformAttributeValue] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["t0", b"t0"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attributes", b"attributes", "dt", b"dt", "signal_count", b"signal_count", "t0", b"t0", "y_data", b"y_data"]) -> None: ...

global___DigitalWaveform = DigitalWaveform

@typing.final
class Scale(google.protobuf.message.Message):
    """Scaling information for converting unscaled waveform data to scaled data."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LINEAR_SCALE_FIELD_NUMBER: builtins.int
    @property
    def linear_scale(self) -> global___LinearScale: ...
    def __init__(
        self,
        *,
        linear_scale: global___LinearScale | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["linear_scale", b"linear_scale", "mode", b"mode"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["linear_scale", b"linear_scale", "mode", b"mode"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["mode", b"mode"]) -> typing.Literal["linear_scale"] | None: ...

global___Scale = Scale

@typing.final
class LinearScale(google.protobuf.message.Message):
    """A linear scale consisting of a gain and an offset."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GAIN_FIELD_NUMBER: builtins.int
    OFFSET_FIELD_NUMBER: builtins.int
    gain: builtins.float
    """The gain of the linear scale"""
    offset: builtins.float
    """The offset of the linear scale"""
    def __init__(
        self,
        *,
        gain: builtins.float = ...,
        offset: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["gain", b"gain", "offset", b"offset"]) -> None: ...

global___LinearScale = LinearScale



"""Methods to convert to and from PrecisionDuration protobuf messages."""

from __future__ import annotations

import hightime as ht
import nitypes.bintime as bt
from nitypes.time import convert_timedelta

from ni.protobuf.types.precision_duration_pb2 import (
    PrecisionDuration,
)


def bintime_timedelta_to_protobuf(value: bt.TimeDelta, /) -> PrecisionDuration:
    """Convert a NI-BTF TimeDelta to a protobuf PrecisionDuration."""
    seconds, fractional_seconds = value.to_tuple()
    return PrecisionDuration(seconds=seconds, fractional_seconds=fractional_seconds)


def bintime_timedelta_from_protobuf(message: PrecisionDuration, /) -> bt.TimeDelta:
    """Convert a protobuf PrecisionDuration to a NI-BTF TimeDelta."""
    time_value_tuple = bt.TimeValueTuple(message.seconds, message.fractional_seconds)
    return bt.TimeDelta.from_tuple(time_value_tuple)


def hightime_timedelta_to_protobuf(value: ht.timedelta, /) -> PrecisionDuration:
    """Convert a hightime.timedelta to a protobuf PrecisionDuration."""
    bt_timedelta = convert_timedelta(bt.TimeDelta, value)
    return bintime_timedelta_to_protobuf(bt_timedelta)


def hightime_timedelta_from_protobuf(message: PrecisionDuration, /) -> ht.timedelta:
    """Convert a protobuf PrecisionDuration to a hightime.timedelta."""
    bt_timedelta = bintime_timedelta_from_protobuf(message)
    ht_timedelta = convert_timedelta(ht.timedelta, bt_timedelta)
    return ht_timedelta



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
---------------------------------------------------------------------
---------------------------------------------------------------------
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import ni.protobuf.types.attribute_value_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class DoubleXYData(google.protobuf.message.Message):
    """XYData for a cartesian graph.

    x_data and y_data should contain the same number of values.
    If they do not, the smaller-sized array will be used to determine
    the number of XY points.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttributesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> ni.protobuf.types.attribute_value_pb2.AttributeValue: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: ni.protobuf.types.attribute_value_pb2.AttributeValue | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    X_DATA_FIELD_NUMBER: builtins.int
    Y_DATA_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    @property
    def x_data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def y_data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def attributes(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, ni.protobuf.types.attribute_value_pb2.AttributeValue]:
        """The names and values of all xy data attributes.

        An attribute is metadata attached to a xy data.
        It is represented in this message as a map associating the name of
        the attribute with the value described by AttributeValue.
        """

    def __init__(
        self,
        *,
        x_data: collections.abc.Iterable[builtins.float] | None = ...,
        y_data: collections.abc.Iterable[builtins.float] | None = ...,
        attributes: collections.abc.Mapping[builtins.str, ni.protobuf.types.attribute_value_pb2.AttributeValue] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["attributes", b"attributes", "x_data", b"x_data", "y_data", b"y_data"]) -> None: ...

global___DoubleXYData = DoubleXYData



"""Methods to convert between ExtendedPropertyDictionaries and AttributeValue maps."""

from __future__ import annotations

from collections.abc import Mapping

from nitypes.waveform import ExtendedPropertyDictionary
from nitypes.waveform.typing import ExtendedPropertyValue

from ni.protobuf.types.attribute_value_pb2 import AttributeValue


def extended_properties_to_protobuf(
    extended_properties: ExtendedPropertyDictionary,
) -> dict[str, AttributeValue]:
    """Convert an ExtendedPropertyDictionary to an AttributeValue map."""
    return {key: _value_to_attribute(value) for key, value in extended_properties.items()}


def _value_to_attribute(value: ExtendedPropertyValue) -> AttributeValue:
    attr_value = AttributeValue()
    if isinstance(value, bool):
        attr_value.bool_value = value
    elif isinstance(value, int):
        attr_value.integer_value = value
    elif isinstance(value, float):
        attr_value.double_value = value
    elif isinstance(value, str):
        attr_value.string_value = value
    else:
        raise TypeError(f"Unexpected type for extended property value {type(value)}")

    return attr_value


def extended_properties_from_protobuf(
    attributes: Mapping[str, AttributeValue],
    extended_properties: ExtendedPropertyDictionary,
) -> None:
    """Convert an AttributeValue map and insert values into an ExtendedPropertyDictionary."""
    for key, value in attributes.items():
        attr_type = value.WhichOneof("attribute")
        if attr_type is None:
            raise ValueError("Could not determine the data type of 'attribute'.")
        extended_properties[key] = getattr(value, attr_type)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ni/protobuf/types/precision_duration.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*ni/protobuf/types/precision_duration.proto\x12\x11ni.protobuf.types\"@\n\x11PrecisionDuration\x12\x0f\n\x07seconds\x18\x01 \x01(\x03\x12\x1a\n\x12\x66ractional_seconds\x18\x02 \x01(\x04\x42\x8e\x01\n\x15\x63om.ni.protobuf.typesB\x16PrecisionDurationProtoP\x01Z\x05types\xa2\x02\x04NIPT\xaa\x02\"NationalInstruments.Protobuf.Types\xca\x02\x11NI\\PROTOBUF\\TYPES\xea\x02\x13NI::Protobuf::Typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ni.protobuf.types.precision_duration_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.ni.protobuf.typesB\026PrecisionDurationProtoP\001Z\005types\242\002\004NIPT\252\002\"NationalInstruments.Protobuf.Types\312\002\021NI\\PROTOBUF\\TYPES\352\002\023NI::Protobuf::Types'
  _PRECISIONDURATION._serialized_start=65
  _PRECISIONDURATION._serialized_end=129
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ni/protobuf/types/vector.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ni.protobuf.types import array_pb2 as ni_dot_protobuf_dot_types_dot_array__pb2
from ni.protobuf.types import attribute_value_pb2 as ni_dot_protobuf_dot_types_dot_attribute__value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eni/protobuf/types/vector.proto\x12\x11ni.protobuf.types\x1a\x1dni/protobuf/types/array.proto\x1a\'ni/protobuf/types/attribute_value.proto\"\x82\x03\n\x06Vector\x12=\n\nattributes\x18\x01 \x03(\x0b\x32).ni.protobuf.types.Vector.AttributesEntry\x12\x36\n\x0c\x64ouble_array\x18\x02 \x01(\x0b\x32\x1e.ni.protobuf.types.DoubleArrayH\x00\x12\x36\n\x0csint32_array\x18\x03 \x01(\x0b\x32\x1e.ni.protobuf.types.SInt32ArrayH\x00\x12\x32\n\nbool_array\x18\x04 \x01(\x0b\x32\x1c.ni.protobuf.types.BoolArrayH\x00\x12\x36\n\x0cstring_array\x18\x05 \x01(\x0b\x32\x1e.ni.protobuf.types.StringArrayH\x00\x1aT\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x30\n\x05value\x18\x02 \x01(\x0b\x32!.ni.protobuf.types.AttributeValue:\x02\x38\x01\x42\x07\n\x05valueB\x83\x01\n\x15\x63om.ni.protobuf.typesB\x0bVectorProtoP\x01Z\x05types\xa2\x02\x04NIPT\xaa\x02\"NationalInstruments.Protobuf.Types\xca\x02\x11NI\\PROTOBUF\\TYPES\xea\x02\x13NI::Protobuf::Typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ni.protobuf.types.vector_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.ni.protobuf.typesB\013VectorProtoP\001Z\005types\242\002\004NIPT\252\002\"NationalInstruments.Protobuf.Types\312\002\021NI\\PROTOBUF\\TYPES\352\002\023NI::Protobuf::Types'
  _VECTOR_ATTRIBUTESENTRY._options = None
  _VECTOR_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _VECTOR._serialized_start=126
  _VECTOR._serialized_end=512
  _VECTOR_ATTRIBUTESENTRY._serialized_start=419
  _VECTOR_ATTRIBUTESENTRY._serialized_end=503
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
---------------------------------------------------------------------
---------------------------------------------------------------------
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import ni.protobuf.types.waveform_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class DoubleAnalogWaveformArrayValue(google.protobuf.message.Message):
    """An array of double-precision analog waveforms."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WAVEFORMS_FIELD_NUMBER: builtins.int
    @property
    def waveforms(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ni.protobuf.types.waveform_pb2.DoubleAnalogWaveform]: ...
    def __init__(
        self,
        *,
        waveforms: collections.abc.Iterable[ni.protobuf.types.waveform_pb2.DoubleAnalogWaveform] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["waveforms", b"waveforms"]) -> None: ...

global___DoubleAnalogWaveformArrayValue = DoubleAnalogWaveformArrayValue

@typing.final
class I16AnalogWaveformArrayValue(google.protobuf.message.Message):
    """An array of 16-bit integer waveforms."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WAVEFORMS_FIELD_NUMBER: builtins.int
    @property
    def waveforms(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ni.protobuf.types.waveform_pb2.I16AnalogWaveform]: ...
    def __init__(
        self,
        *,
        waveforms: collections.abc.Iterable[ni.protobuf.types.waveform_pb2.I16AnalogWaveform] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["waveforms", b"waveforms"]) -> None: ...

global___I16AnalogWaveformArrayValue = I16AnalogWaveformArrayValue

@typing.final
class DoubleComplexWaveformArrayValue(google.protobuf.message.Message):
    """An array of double-precision complex waveforms."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WAVEFORMS_FIELD_NUMBER: builtins.int
    @property
    def waveforms(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ni.protobuf.types.waveform_pb2.DoubleComplexWaveform]: ...
    def __init__(
        self,
        *,
        waveforms: collections.abc.Iterable[ni.protobuf.types.waveform_pb2.DoubleComplexWaveform] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["waveforms", b"waveforms"]) -> None: ...

global___DoubleComplexWaveformArrayValue = DoubleComplexWaveformArrayValue

@typing.final
class I16ComplexWaveformArrayValue(google.protobuf.message.Message):
    """An array of 16-bit integer complex waveforms."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WAVEFORMS_FIELD_NUMBER: builtins.int
    @property
    def waveforms(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ni.protobuf.types.waveform_pb2.I16ComplexWaveform]: ...
    def __init__(
        self,
        *,
        waveforms: collections.abc.Iterable[ni.protobuf.types.waveform_pb2.I16ComplexWaveform] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["waveforms", b"waveforms"]) -> None: ...

global___I16ComplexWaveformArrayValue = I16ComplexWaveformArrayValue

@typing.final
class DoubleSpectrumArrayValue(google.protobuf.message.Message):
    """An array of double-precision spectrums."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WAVEFORMS_FIELD_NUMBER: builtins.int
    @property
    def waveforms(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ni.protobuf.types.waveform_pb2.DoubleSpectrum]: ...
    def __init__(
        self,
        *,
        waveforms: collections.abc.Iterable[ni.protobuf.types.waveform_pb2.DoubleSpectrum] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["waveforms", b"waveforms"]) -> None: ...

global___DoubleSpectrumArrayValue = DoubleSpectrumArrayValue



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import ni.protobuf.types.array_pb2
import ni.protobuf.types.attribute_value_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Vector(google.protobuf.message.Message):
    """A vector value with associated attributes, such as units."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttributesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> ni.protobuf.types.attribute_value_pb2.AttributeValue: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: ni.protobuf.types.attribute_value_pb2.AttributeValue | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    ATTRIBUTES_FIELD_NUMBER: builtins.int
    DOUBLE_ARRAY_FIELD_NUMBER: builtins.int
    SINT32_ARRAY_FIELD_NUMBER: builtins.int
    BOOL_ARRAY_FIELD_NUMBER: builtins.int
    STRING_ARRAY_FIELD_NUMBER: builtins.int
    @property
    def attributes(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, ni.protobuf.types.attribute_value_pb2.AttributeValue]:
        """The names and values of all vector attributes.

        A vector attribute is metadata attached to a vector.
        It is represented in this message as a map associating the name of
        the attribute with the value described by AttributeValue.
        """

    @property
    def double_array(self) -> ni.protobuf.types.array_pb2.DoubleArray: ...
    @property
    def sint32_array(self) -> ni.protobuf.types.array_pb2.SInt32Array: ...
    @property
    def bool_array(self) -> ni.protobuf.types.array_pb2.BoolArray: ...
    @property
    def string_array(self) -> ni.protobuf.types.array_pb2.StringArray: ...
    def __init__(
        self,
        *,
        attributes: collections.abc.Mapping[builtins.str, ni.protobuf.types.attribute_value_pb2.AttributeValue] | None = ...,
        double_array: ni.protobuf.types.array_pb2.DoubleArray | None = ...,
        sint32_array: ni.protobuf.types.array_pb2.SInt32Array | None = ...,
        bool_array: ni.protobuf.types.array_pb2.BoolArray | None = ...,
        string_array: ni.protobuf.types.array_pb2.StringArray | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["bool_array", b"bool_array", "double_array", b"double_array", "sint32_array", b"sint32_array", "string_array", b"string_array", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attributes", b"attributes", "bool_array", b"bool_array", "double_array", b"double_array", "sint32_array", b"sint32_array", "string_array", b"string_array", "value", b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["value", b"value"]) -> typing.Literal["double_array", "sint32_array", "bool_array", "string_array"] | None: ...

global___Vector = Vector



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
---------------------------------------------------------------------
---------------------------------------------------------------------
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class PrecisionDuration(google.protobuf.message.Message):
    """Represents a duration of time expressed in NI Binary Time Format (NI-BTF).

    https://www.ni.com/docs/en-US/bundle/labwindows-cvi/page/cvi/libref/ni-btf.htm
    A PrecisionDuration is encoded as a count of seconds and fractions of seconds at
    2^-64 resolution.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECONDS_FIELD_NUMBER: builtins.int
    FRACTIONAL_SECONDS_FIELD_NUMBER: builtins.int
    seconds: builtins.int
    """The number of seconds."""
    fractional_seconds: builtins.int
    """Non-negative fractions of a second at 2^-64 resolution."""
    def __init__(
        self,
        *,
        seconds: builtins.int = ...,
        fractional_seconds: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["fractional_seconds", b"fractional_seconds", "seconds", b"seconds"]) -> None: ...

global___PrecisionDuration = PrecisionDuration



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import ni.protobuf.types.attribute_value_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Scalar(google.protobuf.message.Message):
    """A scalar value with associated attributes, such as units."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttributesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> ni.protobuf.types.attribute_value_pb2.AttributeValue: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: ni.protobuf.types.attribute_value_pb2.AttributeValue | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    ATTRIBUTES_FIELD_NUMBER: builtins.int
    DOUBLE_VALUE_FIELD_NUMBER: builtins.int
    SINT32_VALUE_FIELD_NUMBER: builtins.int
    BOOL_VALUE_FIELD_NUMBER: builtins.int
    STRING_VALUE_FIELD_NUMBER: builtins.int
    double_value: builtins.float
    sint32_value: builtins.int
    bool_value: builtins.bool
    string_value: builtins.str
    @property
    def attributes(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, ni.protobuf.types.attribute_value_pb2.AttributeValue]:
        """The names and values of all scalar attributes.

        A scalar attribute is metadata attached to a scalar.
        It is represented in this message as a map associating the name of
        the attribute with the value described by AttributeValue.
        """

    def __init__(
        self,
        *,
        attributes: collections.abc.Mapping[builtins.str, ni.protobuf.types.attribute_value_pb2.AttributeValue] | None = ...,
        double_value: builtins.float = ...,
        sint32_value: builtins.int = ...,
        bool_value: builtins.bool = ...,
        string_value: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["bool_value", b"bool_value", "double_value", b"double_value", "sint32_value", b"sint32_value", "string_value", b"string_value", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attributes", b"attributes", "bool_value", b"bool_value", "double_value", b"double_value", "sint32_value", b"sint32_value", "string_value", b"string_value", "value", b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["value", b"value"]) -> typing.Literal["double_value", "sint32_value", "bool_value", "string_value"] | None: ...

global___Scalar = Scalar



"""Methods to convert to and from scalar protobuf messages."""

from __future__ import annotations

from typing import Union

from nitypes.scalar import Scalar
from typing_extensions import TypeAlias

import ni.protobuf.types.scalar_pb2 as scalar_pb2
from ni.protobuf.types.extended_property_conversion import (
    extended_properties_from_protobuf,
    extended_properties_to_protobuf,
)

AnyScalarType: TypeAlias = Union[bool, int, float, str]
_SCALAR_TYPE_TO_PB_ATTR_MAP = {
    bool: "bool_value",
    int: "sint32_value",
    float: "double_value",
    str: "string_value",
}


def scalar_to_protobuf(value: Scalar[AnyScalarType], /) -> scalar_pb2.Scalar:
    """Convert a Scalar python object to a protobuf scalar_pb2.Scalar."""
    attributes = extended_properties_to_protobuf(value.extended_properties)
    message = scalar_pb2.Scalar(attributes=attributes)

    # Convert the scalar value
    _check_scalar_value(value.value)
    value_attr = _SCALAR_TYPE_TO_PB_ATTR_MAP.get(type(value.value), None)
    if not value_attr:
        raise TypeError(f"Unexpected type for value.value: {type(value.value)}")

    setattr(message, value_attr, value.value)

    return message


def scalar_from_protobuf(message: scalar_pb2.Scalar, /) -> Scalar[AnyScalarType]:
    """Convert the protobuf scalar_pb2.Scalar to a Python Scalar."""
    # Convert the scalar value.
    pb_type = message.WhichOneof("value")
    if pb_type is None:
        raise ValueError("Could not determine the data type of 'value'.")

    if pb_type not in _SCALAR_TYPE_TO_PB_ATTR_MAP.values():
        raise ValueError(f"Unexpected value for protobuf_value.WhichOneOf: {pb_type}")
    value = getattr(message, pb_type)

    # Create with blank units. Units from the protobuf message will be populated
    # when attributes are converted to an ExtendedPropertyDictionary.
    scalar = Scalar(value, "")

    # Transfer attributes to extended_properties
    extended_properties_from_protobuf(message.attributes, scalar.extended_properties)

    return scalar


def _check_scalar_value(value: AnyScalarType) -> None:
    """Perform value checking on a scalar value."""
    if isinstance(value, int):
        if value <= -0x80000000 or value >= 0x7FFFFFFF:
            raise ValueError("The integer value in a scalar must be within the range of an Int32.")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ni/protobuf/types/xydata.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ni.protobuf.types import attribute_value_pb2 as ni_dot_protobuf_dot_types_dot_attribute__value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eni/protobuf/types/xydata.proto\x12\x11ni.protobuf.types\x1a\'ni/protobuf/types/attribute_value.proto\"\xc9\x01\n\x0c\x44oubleXYData\x12\x0e\n\x06x_data\x18\x01 \x03(\x01\x12\x0e\n\x06y_data\x18\x02 \x03(\x01\x12\x43\n\nattributes\x18\x03 \x03(\x0b\x32/.ni.protobuf.types.DoubleXYData.AttributesEntry\x1aT\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x30\n\x05value\x18\x02 \x01(\x0b\x32!.ni.protobuf.types.AttributeValue:\x02\x38\x01\x42\x83\x01\n\x15\x63om.ni.protobuf.typesB\x0bXYDataProtoP\x01Z\x05types\xa2\x02\x04NIPT\xaa\x02\"NationalInstruments.Protobuf.Types\xca\x02\x11NI\\PROTOBUF\\TYPES\xea\x02\x13NI::Protobuf::Typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ni.protobuf.types.xydata_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.ni.protobuf.typesB\013XYDataProtoP\001Z\005types\242\002\004NIPT\252\002\"NationalInstruments.Protobuf.Types\312\002\021NI\\PROTOBUF\\TYPES\352\002\023NI::Protobuf::Types'
  _DOUBLEXYDATA_ATTRIBUTESENTRY._options = None
  _DOUBLEXYDATA_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _DOUBLEXYDATA._serialized_start=95
  _DOUBLEXYDATA._serialized_end=296
  _DOUBLEXYDATA_ATTRIBUTESENTRY._serialized_start=212
  _DOUBLEXYDATA_ATTRIBUTESENTRY._serialized_end=296
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ni/protobuf/types/array.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dni/protobuf/types/array.proto\x12\x11ni.protobuf.types\"\x1b\n\tBoolArray\x12\x0e\n\x06values\x18\x01 \x03(\x08\"\x1c\n\nBytesArray\x12\x0e\n\x06values\x18\x01 \x03(\x0c\"\x1d\n\x0b\x44oubleArray\x12\x0e\n\x06values\x18\x01 \x03(\x01\"\x1d\n\x0bSInt32Array\x12\x0e\n\x06values\x18\x01 \x03(\x11\"\x1d\n\x0bSInt64Array\x12\x0e\n\x06values\x18\x01 \x03(\x12\"\x1d\n\x0bStringArray\x12\x0e\n\x06values\x18\x01 \x03(\t\"<\n\rDouble2DArray\x12\x0c\n\x04rows\x18\x01 \x01(\x05\x12\x0f\n\x07\x63olumns\x18\x02 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\x01\"<\n\rString2DArray\x12\x0c\n\x04rows\x18\x01 \x01(\x05\x12\x0f\n\x07\x63olumns\x18\x02 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\tB\x82\x01\n\x15\x63om.ni.protobuf.typesB\nArrayProtoP\x01Z\x05types\xa2\x02\x04NIPT\xaa\x02\"NationalInstruments.Protobuf.Types\xca\x02\x11NI\\PROTOBUF\\TYPES\xea\x02\x13NI::Protobuf::Typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ni.protobuf.types.array_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.ni.protobuf.typesB\nArrayProtoP\001Z\005types\242\002\004NIPT\252\002\"NationalInstruments.Protobuf.Types\312\002\021NI\\PROTOBUF\\TYPES\352\002\023NI::Protobuf::Types'
  _BOOLARRAY._serialized_start=52
  _BOOLARRAY._serialized_end=79
  _BYTESARRAY._serialized_start=81
  _BYTESARRAY._serialized_end=109
  _DOUBLEARRAY._serialized_start=111
  _DOUBLEARRAY._serialized_end=140
  _SINT32ARRAY._serialized_start=142
  _SINT32ARRAY._serialized_end=171
  _SINT64ARRAY._serialized_start=173
  _SINT64ARRAY._serialized_end=202
  _STRINGARRAY._serialized_start=204
  _STRINGARRAY._serialized_end=233
  _DOUBLE2DARRAY._serialized_start=235
  _DOUBLE2DARRAY._serialized_end=295
  _STRING2DARRAY._serialized_start=297
  _STRING2DARRAY._serialized_end=357
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ni/protobuf/types/waveform.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ni.protobuf.types import precision_timestamp_pb2 as ni_dot_protobuf_dot_types_dot_precision__timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n ni/protobuf/types/waveform.proto\x12\x11ni.protobuf.types\x1a+ni/protobuf/types/precision_timestamp.proto\"\x90\x02\n\x14\x44oubleAnalogWaveform\x12\x31\n\x02t0\x18\x01 \x01(\x0b\x32%.ni.protobuf.types.PrecisionTimestamp\x12\n\n\x02\x64t\x18\x02 \x01(\x01\x12\x0e\n\x06y_data\x18\x03 \x03(\x01\x12K\n\nattributes\x18\x04 \x03(\x0b\x32\x37.ni.protobuf.types.DoubleAnalogWaveform.AttributesEntry\x1a\\\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x38\n\x05value\x18\x02 \x01(\x0b\x32).ni.protobuf.types.WaveformAttributeValue:\x02\x38\x01\"\xb3\x02\n\x11I16AnalogWaveform\x12\x31\n\x02t0\x18\x01 \x01(\x0b\x32%.ni.protobuf.types.PrecisionTimestamp\x12\n\n\x02\x64t\x18\x02 \x01(\x01\x12\x0e\n\x06y_data\x18\x03 \x03(\x11\x12H\n\nattributes\x18\x04 \x03(\x0b\x32\x34.ni.protobuf.types.I16AnalogWaveform.AttributesEntry\x12\'\n\x05scale\x18\x05 \x01(\x0b\x32\x18.ni.protobuf.types.Scale\x1a\\\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x38\n\x05value\x18\x02 \x01(\x0b\x32).ni.protobuf.types.WaveformAttributeValue:\x02\x38\x01\"\x92\x02\n\x15\x44oubleComplexWaveform\x12\x31\n\x02t0\x18\x01 \x01(\x0b\x32%.ni.protobuf.types.PrecisionTimestamp\x12\n\n\x02\x64t\x18\x02 \x01(\x01\x12\x0e\n\x06y_data\x18\x03 \x03(\x01\x12L\n\nattributes\x18\x04 \x03(\x0b\x32\x38.ni.protobuf.types.DoubleComplexWaveform.AttributesEntry\x1a\\\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x38\n\x05value\x18\x02 \x01(\x0b\x32).ni.protobuf.types.WaveformAttributeValue:\x02\x38\x01\"\xb5\x02\n\x12I16ComplexWaveform\x12\x31\n\x02t0\x18\x01 \x01(\x0b\x32%.ni.protobuf.types.PrecisionTimestamp\x12\n\n\x02\x64t\x18\x02 \x01(\x01\x12\x0e\n\x06y_data\x18\x03 \x03(\x11\x12I\n\nattributes\x18\x04 \x03(\x0b\x32\x35.ni.protobuf.types.I16ComplexWaveform.AttributesEntry\x12\'\n\x05scale\x18\x05 \x01(\x0b\x32\x18.ni.protobuf.types.Scale\x1a\\\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x38\n\x05value\x18\x02 \x01(\x0b\x32).ni.protobuf.types.WaveformAttributeValue:\x02\x38\x01\"\xf9\x01\n\x0e\x44oubleSpectrum\x12\x17\n\x0fstart_frequency\x18\x01 \x01(\x01\x12\x1b\n\x13\x66requency_increment\x18\x02 \x01(\x01\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\x01\x12\x45\n\nattributes\x18\x04 \x03(\x0b\x32\x31.ni.protobuf.types.DoubleSpectrum.AttributesEntry\x1a\\\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x38\n\x05value\x18\x02 \x01(\x0b\x32).ni.protobuf.types.WaveformAttributeValue:\x02\x38\x01\"\x84\x01\n\x16WaveformAttributeValue\x12\x14\n\nbool_value\x18\x01 \x01(\x08H\x00\x12\x17\n\rinteger_value\x18\x02 \x01(\x05H\x00\x12\x16\n\x0c\x64ouble_value\x18\x03 \x01(\x01H\x00\x12\x16\n\x0cstring_value\x18\x04 \x01(\tH\x00\x42\x0b\n\tattribute\"\x9c\x02\n\x0f\x44igitalWaveform\x12\x31\n\x02t0\x18\x01 \x01(\x0b\x32%.ni.protobuf.types.PrecisionTimestamp\x12\n\n\x02\x64t\x18\x02 \x01(\x01\x12\x14\n\x0csignal_count\x18\x03 \x01(\x05\x12\x0e\n\x06y_data\x18\x04 \x01(\x0c\x12\x46\n\nattributes\x18\x05 \x03(\x0b\x32\x32.ni.protobuf.types.DigitalWaveform.AttributesEntry\x1a\\\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x38\n\x05value\x18\x02 \x01(\x0b\x32).ni.protobuf.types.WaveformAttributeValue:\x02\x38\x01\"G\n\x05Scale\x12\x36\n\x0clinear_scale\x18\x01 \x01(\x0b\x32\x1e.ni.protobuf.types.LinearScaleH\x00\x42\x06\n\x04mode\"+\n\x0bLinearScale\x12\x0c\n\x04gain\x18\x01 \x01(\x01\x12\x0e\n\x06offset\x18\x02 \x01(\x01\x42\x85\x01\n\x15\x63om.ni.protobuf.typesB\rWaveformProtoP\x01Z\x05types\xa2\x02\x04NIPT\xaa\x02\"NationalInstruments.Protobuf.Types\xca\x02\x11NI\\PROTOBUF\\TYPES\xea\x02\x13NI::Protobuf::Typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ni.protobuf.types.waveform_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.ni.protobuf.typesB\rWaveformProtoP\001Z\005types\242\002\004NIPT\252\002\"NationalInstruments.Protobuf.Types\312\002\021NI\\PROTOBUF\\TYPES\352\002\023NI::Protobuf::Types'
  _DOUBLEANALOGWAVEFORM_ATTRIBUTESENTRY._options = None
  _DOUBLEANALOGWAVEFORM_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _I16ANALOGWAVEFORM_ATTRIBUTESENTRY._options = None
  _I16ANALOGWAVEFORM_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _DOUBLECOMPLEXWAVEFORM_ATTRIBUTESENTRY._options = None
  _DOUBLECOMPLEXWAVEFORM_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _I16COMPLEXWAVEFORM_ATTRIBUTESENTRY._options = None
  _I16COMPLEXWAVEFORM_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _DOUBLESPECTRUM_ATTRIBUTESENTRY._options = None
  _DOUBLESPECTRUM_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _DIGITALWAVEFORM_ATTRIBUTESENTRY._options = None
  _DIGITALWAVEFORM_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _DOUBLEANALOGWAVEFORM._serialized_start=101
  _DOUBLEANALOGWAVEFORM._serialized_end=373
  _DOUBLEANALOGWAVEFORM_ATTRIBUTESENTRY._serialized_start=281
  _DOUBLEANALOGWAVEFORM_ATTRIBUTESENTRY._serialized_end=373
  _I16ANALOGWAVEFORM._serialized_start=376
  _I16ANALOGWAVEFORM._serialized_end=683
  _I16ANALOGWAVEFORM_ATTRIBUTESENTRY._serialized_start=281
  _I16ANALOGWAVEFORM_ATTRIBUTESENTRY._serialized_end=373
  _DOUBLECOMPLEXWAVEFORM._serialized_start=686
  _DOUBLECOMPLEXWAVEFORM._serialized_end=960
  _DOUBLECOMPLEXWAVEFORM_ATTRIBUTESENTRY._serialized_start=281
  _DOUBLECOMPLEXWAVEFORM_ATTRIBUTESENTRY._serialized_end=373
  _I16COMPLEXWAVEFORM._serialized_start=963
  _I16COMPLEXWAVEFORM._serialized_end=1272
  _I16COMPLEXWAVEFORM_ATTRIBUTESENTRY._serialized_start=281
  _I16COMPLEXWAVEFORM_ATTRIBUTESENTRY._serialized_end=373
  _DOUBLESPECTRUM._serialized_start=1275
  _DOUBLESPECTRUM._serialized_end=1524
  _DOUBLESPECTRUM_ATTRIBUTESENTRY._serialized_start=281
  _DOUBLESPECTRUM_ATTRIBUTESENTRY._serialized_end=373
  _WAVEFORMATTRIBUTEVALUE._serialized_start=1527
  _WAVEFORMATTRIBUTEVALUE._serialized_end=1659
  _DIGITALWAVEFORM._serialized_start=1662
  _DIGITALWAVEFORM._serialized_end=1946
  _DIGITALWAVEFORM_ATTRIBUTESENTRY._serialized_start=281
  _DIGITALWAVEFORM_ATTRIBUTESENTRY._serialized_end=373
  _SCALE._serialized_start=1948
  _SCALE._serialized_end=2019
  _LINEARSCALE._serialized_start=2021
  _LINEARSCALE._serialized_end=2064
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
---------------------------------------------------------------------
---------------------------------------------------------------------
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class BoolArray(google.protobuf.message.Message):
    """An array of booleans"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bool]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[builtins.bool] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___BoolArray = BoolArray

@typing.final
class BytesArray(google.protobuf.message.Message):
    """An array of bytes collections"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[builtins.bytes] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___BytesArray = BytesArray

@typing.final
class DoubleArray(google.protobuf.message.Message):
    """An array of doubles"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___DoubleArray = DoubleArray

@typing.final
class SInt32Array(google.protobuf.message.Message):
    """An array of 32-bit integers"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___SInt32Array = SInt32Array

@typing.final
class SInt64Array(google.protobuf.message.Message):
    """An array of 64-bit integers"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___SInt64Array = SInt64Array

@typing.final
class StringArray(google.protobuf.message.Message):
    """An array of strings"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___StringArray = StringArray

@typing.final
class Double2DArray(google.protobuf.message.Message):
    """A 2D array of doubles.

    2D arrays are stored as a repeated field of the appropriate element type,
    a 1D array. Values are stored in row major order.

    Example:
    Repeated Double: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]

    rows: 2
    columns: 5

    2D Representation:
     1  2  3  4  5
     6  7  8  9 10

    Indices:
    (0,0) (0,1) (0,2) (0,3) (0,4)
    (1,0) (1,1) (1,2) (1,3) (1,4)

    Remarks:
     The length of the 'data' field must be equal to rows * columns.
     If it is not, implementations should treat this state as invalid
     and return INVALID_ARGUMENT status code if appropriate.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROWS_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    rows: builtins.int
    columns: builtins.int
    @property
    def data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        rows: builtins.int = ...,
        columns: builtins.int = ...,
        data: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["columns", b"columns", "data", b"data", "rows", b"rows"]) -> None: ...

global___Double2DArray = Double2DArray

@typing.final
class String2DArray(google.protobuf.message.Message):
    """A 2D array of strings."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROWS_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    rows: builtins.int
    columns: builtins.int
    @property
    def data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        rows: builtins.int = ...,
        columns: builtins.int = ...,
        data: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["columns", b"columns", "data", b"data", "rows", b"rows"]) -> None: ...

global___String2DArray = String2DArray



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class AttributeValue(google.protobuf.message.Message):
    """An attribute that can be attached to a scalar, vector, or similar message."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BOOL_VALUE_FIELD_NUMBER: builtins.int
    INTEGER_VALUE_FIELD_NUMBER: builtins.int
    DOUBLE_VALUE_FIELD_NUMBER: builtins.int
    STRING_VALUE_FIELD_NUMBER: builtins.int
    bool_value: builtins.bool
    """Represents a bool attribute."""
    integer_value: builtins.int
    """Represents an integer attribute."""
    double_value: builtins.float
    """Represents a double attribute."""
    string_value: builtins.str
    """Represents a string attribute."""
    def __init__(
        self,
        *,
        bool_value: builtins.bool = ...,
        integer_value: builtins.int = ...,
        double_value: builtins.float = ...,
        string_value: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["attribute", b"attribute", "bool_value", b"bool_value", "double_value", b"double_value", "integer_value", b"integer_value", "string_value", b"string_value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attribute", b"attribute", "bool_value", b"bool_value", "double_value", b"double_value", "integer_value", b"integer_value", "string_value", b"string_value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["attribute", b"attribute"]) -> typing.Literal["bool_value", "integer_value", "double_value", "string_value"] | None: ...

global___AttributeValue = AttributeValue
